*-----------------------------------------------------------
* Program    : Echo Server v2.0
* Written by : Charles Kelly
* Date       : July-25-2014
* Description: For EASy68K v5.15.0 or later
* Port 48161
* Displays incomming message and echos it back to sender.
* In TCP mode only one client at a time may communicate.
* In UDP mode multiple clients may connect.
*-----------------------------------------------------------

* Display string
PRINT   MACRO
    ifnc    \1,A1           ; if first parameter is not A1
    lea     \1,A1           ; get string name from macro parameter
    endc
    move.b  #14,d0
    trap    #15             ; display string at (A1) without newline
    ENDM

* Clear the screen
CLEAR_SCREEN  MACRO
    movem.l d0/d1,-(sp)
    move.b  #11,d0
    move.w  #$FF00,d1
    trap    #15             ; clear screen
    movem.l (sp)+,d0/d1
    ENDM

* Display network error
PRINT_ERROR     MACRO
    movem.l d0/a1,-(sp)
    and.l   #$0ffff,d0          ; remove extended error code
    sub.l   #1,d0               ; make error code 0-7
    mulu.w  #netErrWidth,d0     ; calculate offset to error message
    lea     netErrors,a1
    lea     (a1,d0),a1
    PRINT   A1
    movem.l (sp)+,d0/a1
    ENDM


	ORG	$1000
START:				        ; first instruction of program

    ; check EASy68K version
    move.b  #32,d0
    move.b  #4,d1
    trap    #15             ; version number returned in D1.L
    if.l d1 <lo> #$050F00 then.s    ; if older than version 5.15.0
        PRINT   versionError
        bra     halt
    endi

    ; ***** create server *****
	clr.l	d1
    repeat
        PRINT   protocolMnu
        move.b  #5,d0           ; get character
        trap    #15
    until.b d1 <eq> #'0' or.b d1 <eq> #'1' do.s
    sub.b   #$30,d1         ; convert '0' or '1' character to digit
    move.w  d1,protocol     ; save protocol

    move.b  #101,d0       
    move.w  #48161,d1       ; port 48161
    swap    d1
    move.w  protocol,d1     ; UDP or TCP
    trap    #15
    if.l D0 <ne> #0 then.s  ; if error
        PRINT_ERROR
        bra     halt
    endi
    ; display serverIP
    move.b  #105,d0
    lea     localIP,a2
    trap    #15             ; get local IP
    PRINT   serverIPmsg
    move.l  a2,a1
    PRINT   A1              ; print IP address at (A1)
    PRINT   CRLF

loop
    ; display incomming text and port and echo back
    ; check for receive
    move.b  #107,d0         ; receive data
    lea     buf,a1          ; receive buffer
    lea     remoteIP,a2     ; IP address of sender
    move.l  #BUFSIZE,d1     ; max chars to read
    trap    #15

	if.l d0 <ne> #0 then.s	; if network error
		repeat
			move.b	#104,d0	; close connection
			trap	#15
		until.l d0 <eq> #0 do.s
        move.b  #101,d0       	; re-create server
        move.w  #48161,d1       ; port 48161
        swap    d1
        move.w  protocol,d1     ; UDP or TCP
        trap    #15
        if.l D0 <ne> #0 then.s  ; if error
            PRINT_ERROR
            bra     halt
        endi
        clr.l	d1
	endi

    if.w d0 <eq> #0 and.w d1 <hi> #0 then.l  ; if characters received
        ; display incomming message
        lea     messageRcv,a1   ; display message port
        move.b  #17,d0
        move.l  d1,d2       ; port number and number of bytes received
        clr.w   d1
        swap    d1          ; port number in d1
        trap    #15
        lea     CRLF,a1     ; display newline
        move.b  #14,d0
        trap    #15
        move.w  d2,d1       ; number of characters received
        
        move.b  #0,d0       ; display d1 chars with CR,LF
        lea     buf,a1
        trap    #15
        ; echo message back to client on port
        move.b  #106,d0     ; send data
        move.l  d2,d1       ; port number and number of bytes to send
        lea     buf,a1      ; (a1) data to send
        lea     remoteIP,a2 ; (a2) IP address of client if we are server
        trap    #15
    endi

    ; insert short delay to reduce CPU workload
    move.b  #23,d0          ; delay
    move.l  #1,d1           ; 1/100 second
    trap    #15
    
    bra     loop     

* Communications error
Fail:
    lea     ERROR,A1
    move.b  #13,d0
    trap    #15                 ; display error message

Halt 
    MOVE.B	#9,D0
	TRAP	#15		            ; halt simulator

* Constants
CR          equ $D              ; Carriage Return
LF          equ $A              ; Line Feed
BS          equ $8              ; Backspace
DARK_BLUE   equ $00800000
BLACK       equ $00000000
BUFSIZE     equ 80
BLUE	    equ	$00FF0000


* Variables and Strings
      
        ds.w    0           ; force even
bufIP   ds.b    20

remoteIP    ds.b  16        ; remote IP as string 'aaa.bbb.ccc.ddd',0
localIP     ds.b  16        ; local IP as string 'aaa.bbb.ccc.ddd',0
        
            ds.w  0         ; force even
protocol    ds.w  1         ; 0 for UDP, 1 for TCP

        ds.w    0           ; force word alignment
buf     ds.b    BUFSIZE+1

CRLF            dc.b    CR,LF,0
ERROR           dc.b    CR,LF,'Communications Error',CR,LF,0
serverIPmsg     dc.b    CR,LF,'Server IP is: ',0
versionError    dc.b    'This program requires EASy68K v5.15.0 or newer.',CR,LF,0
protocolMnu     dc.b    CR,LF,'Select Protocol',CR,LF
                dc.b    '0 = UDP',CR,LF
                dc.b    '1 = TCP',CR,LF,CR,LF
                dc.b    'Choice: ',0
messageRcv      dc.b    'Message receive on port: ',0

* All netErrors messages must be 23 characters + 1 null for total of 24
netErrWidth equ 24
netErrors
        dc.b    'General network error  ',0
        dc.b    'Network init failed    ',0
        dc.b    'Invalid socket         ',0
        dc.b    'Get host by name failed',0
        dc.b    'Bind failed            ',0
        dc.b    'Connect failed         ',0
        dc.b    'Port alreay in use     ',0
        dc.b    'Domain not found       ',0

    END	START		; last line of source


































*~Font name~Courier New~
*~Font size~10~
*~Tab type~1~
*~Tab size~4~
